from __future__ import annotations

from pydantic import BaseModel, Field, field_validator

from ambr.utils import remove_html_tags

__all__ = ("Book", "BookDetail", "BookVolume")


class BookVolume(BaseModel):
    """
    Represents a book volume.

    Attributes:
        id (int): The book volume's ID.
        name (str): The book volume's name.
        description (str): The book volume's description.
        story_id (int): The book volume's story ID.
    """

    id: int
    name: str
    description: str
    story_id: int = Field(alias="storyId")

    @field_validator("description", mode="before")
    @classmethod
    def __format_description(cls, v: str) -> str:
        return remove_html_tags(v)


class BookDetail(BaseModel):
    """
    Represents a book detail.

    Attributes:
        id (int): The book's ID.
        name (str): The book's name.
        rarity (int): The book's rarity.
        icon (str): The book's icon.
        volumes (list[BookVolume]): The book's volumes.
    """

    id: int
    name: str
    rarity: int = Field(alias="rank")
    icon: str
    volumes: list[BookVolume] = Field(alias="volume")

    @field_validator("icon", mode="before")
    def _convert_icon_url(cls, v: str) -> str:
        return f"https://gi.yatta.moe/assets/UI/{v}.png"


class Book(BaseModel):
    """
    Represents a book.

    Attributes:
        id (int): The book's ID.
        name (str): The book's name.
        rarity (int): The book's rarity.
        icon (str): The book's icon.
        route (str): The book's route.
    """

    id: int
    name: str
    rarity: int = Field(alias="rank")
    icon: str
    route: str

    @field_validator("icon", mode="before")
    def _convert_icon_url(cls, v: str) -> str:
        return f"https://gi.yatta.moe/assets/UI/{v}.png"



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, field_validator

from ..enums import Element, WeaponType

__all__ = (
    "AvailableItems",
    "AzaBestArtifact",
    "AzaBestArtifactSets",
    "AzaBestItem",
    "AzaData",
    "CharacterGuide",
    "GWBuild",
    "GWBuildInfo",
    "GWBuildInfoNormalArtifact",
    "GWData",
    "GWPlaystyle",
    "GWSynergy",
    "GWSynergyElementCharacter",
    "GWSynergyFlexibleCharacter",
    "GWSynergyInfo",
    "GWSynergyNormalCharacter",
    "GuideArtifact",
    "GuideCharacter",
    "GuideWeapon",
    "GwBuildInfoCustomArtifact",
)


class GuideCharacter(BaseModel):
    id: int
    rarity: Literal[4, 5] = Field(alias="rank")
    weapon_type: WeaponType = Field(alias="weaponType")
    icon: str
    route: str


class GuideWeapon(BaseModel):
    id: int
    rarity: Literal[1, 2, 3, 4, 5] = Field(alias="rank")
    type: WeaponType
    icon: str
    route: str


class GuideArtifact(BaseModel):
    id: int
    icon: str
    rarities: list[int] = Field(alias="levelList")
    route: str


class AvailableItems(BaseModel):
    characters: dict[str, GuideCharacter] = Field(alias="avatar")
    weapons: dict[str, GuideWeapon] = Field(alias="weapon")
    artifacts: dict[str, GuideArtifact] = Field(alias="reliquary")


class GWBuildInfoNormalArtifact(BaseModel):
    id: int
    type: Literal["normal"]


class GwBuildInfoCustomArtifact(BaseModel):
    id: str
    type: Literal["custom"]


class GWBuildInfo(BaseModel):
    inline: bool
    name: str
    value: str | None = None
    weapons: dict[int, str] | None = Field(None, alias="weaponList")
    """Weapon ID to weapon name."""
    artifacts: list[GWBuildInfoNormalArtifact | GwBuildInfoCustomArtifact] | None = Field(
        None, alias="reliquaryList"
    )


class GWBuild(BaseModel):
    """Genshin Wizard build."""

    title: str
    credits: str
    info: list[GWBuildInfo]


class GWPlaystyle(BaseModel):
    title: str
    description: str
    credits: str


class GWSynergyNormalCharacter(BaseModel):
    id: int
    type: Literal["normal"]


class GWSynergyElementCharacter(BaseModel):
    element: Element
    type: Literal["element"]


class GWSynergyFlexibleCharacter(BaseModel):
    type: Literal["flexible"]


class GWSynergyInfo(BaseModel):
    inline: bool
    name: str
    value: str


class GWSynergy(BaseModel):
    title: str
    info: list[GWSynergyInfo] | None = None
    teams: list[
        list[GWSynergyNormalCharacter | GWSynergyElementCharacter | GWSynergyFlexibleCharacter]
    ] = Field(alias="synergiestList")
    credits: str


class GWData(BaseModel):
    builds: list[GWBuild]
    playstyle: GWPlaystyle | None = None
    synergies: GWSynergy


class AzaBestItem(BaseModel):
    id: int
    value: float

    @property
    def percentage(self) -> str:
        return f"{self.value * 100:.1f}%"


class AzaBestArtifact(BaseModel):
    id: int
    num: int


class AzaBestArtifactSets(BaseModel):
    sets: list[AzaBestArtifact] = Field(alias="setList")
    value: float

    @field_validator("sets", mode="before")
    @classmethod
    def __transform_sets(cls, value: dict[str, int]) -> list[AzaBestArtifact]:
        return [AzaBestArtifact(id=int(k), num=v) for k, v in value.items()]

    @property
    def percentage(self) -> str:
        return f"{self.value * 100:.1f}%"


class AzaData(BaseModel):
    best_characters: dict[str, AzaBestItem] = Field(alias="bestAvatarList")
    best_weapons: dict[str, AzaBestItem] = Field(alias="bestWeaponList")
    best_artifact_sets: list[AzaBestArtifactSets] = Field(alias="bestReliquaryList")
    constellation_usage: dict[str, float] = Field(alias="constellationsUsage")


class CharacterGuide(BaseModel):
    available_items: AvailableItems = Field(alias="dataList")
    gw_data: GWData | None = Field(None, alias="gwData")
    """Genshin Wizard data."""
    aza_data: AzaData | None = Field(None, alias="azaData")
    """genshin.aza.gg data."""



from __future__ import annotations

import ambr


async def test_fetch_manual_weapon() -> None:
    async with ambr.AmbrAPI() as api:
        data = await api.fetch_manual_weapon()
        assert isinstance(data, dict)


async def test_fetch_readable() -> None:
    async with ambr.AmbrAPI() as api:
        data = await api.fetch_readable("Weapon11509")
        assert isinstance(data, str)


async def test_fetch_avatar_curve() -> None:
    async with ambr.AmbrAPI() as api:
        data = await api.fetch_avatar_curve()
        assert isinstance(data, dict)


async def test_fetch_weapon_curve() -> None:
    async with ambr.AmbrAPI() as api:
        data = await api.fetch_weapon_curve()
        assert isinstance(data, dict)


async def test_fetch_monster_curve() -> None:
    async with ambr.AmbrAPI() as api:
        data = await api.fetch_monster_curve()
        assert isinstance(data, dict)



from __future__ import annotations

import ambr


async def test_languages() -> None:
    async with ambr.AmbrAPI() as client:
        for lang in ambr.Language:
            client.lang = lang
            await client.fetch_characters()



from __future__ import annotations

from enum import IntEnum, StrEnum

__all__ = ("Element", "ExtraLevelType", "SpecialStat", "TalentType", "WeaponType")


class WeaponType(StrEnum):
    """Character weapon types."""

    BOW = "WEAPON_BOW"
    CATALYST = "WEAPON_CATALYST"
    CLAYMORE = "WEAPON_CLAYMORE"
    SWORD = "WEAPON_SWORD_ONE_HAND"
    POLE = "WEAPON_POLE"


class SpecialStat(StrEnum):
    """Character specialized stat."""

    CRIT_RATE = "FIGHT_PROP_CRITICAL"
    CRITI_DMG = "FIGHT_PROP_CRITICAL_HURT"

    ATTACK = "FIGHT_PROP_ATTACK_PERCENT"
    HP = "FIGHT_PROP_HP_PERCENT"
    DEFENSE = "FIGHT_PROP_DEFENSE_PERCENT"

    HEAL_BONUS = "FIGHT_PROP_HEAL_ADD"
    ELEMENTAL_MASTERY = "FIGHT_PROP_ELEMENT_MASTERY"
    ENERGY_RECHARGE = "FIGHT_PROP_CHARGE_EFFICIENCY"

    HYDRO_DMG_BONUS = "FIGHT_PROP_WATER_ADD_HURT"
    PYRO_DMG_BONUS = "FIGHT_PROP_FIRE_ADD_HURT"
    ELECTRO_DMG_BONUS = "FIGHT_PROP_ELEC_ADD_HURT"
    ANEMO_DMG_BONUS = "FIGHT_PROP_WIND_ADD_HURT"
    CRYO_DMG_BONUS = "FIGHT_PROP_ICE_ADD_HURT"
    GEO_DMG_BONUS = "FIGHT_PROP_ROCK_ADD_HURT"
    DENDRO_DMG_BONUS = "FIGHT_PROP_GRASS_ADD_HURT"
    PHYSICAL_DMG_BONUS = "FIGHT_PROP_PHYSICAL_ADD_HURT"


class Element(StrEnum):
    ANEMO = "Wind"
    GEO = "Rock"
    ELECTRO = "Electric"
    PYRO = "Fire"
    HYDRO = "Water"
    CRYO = "Ice"
    DENDRO = "Grass"


class TalentType(IntEnum):
    NORMAL = 0
    ULTIMATE = 1
    PASSIVE = 2


class ExtraLevelType(IntEnum):
    NORMAL = 1
    ULTIMATE = 9
    SKILL = 2



from __future__ import annotations

import pytest

import ambr


async def test_book() -> None:
    async with ambr.AmbrAPI() as api:
        books = await api.fetch_books()
        for book in books:
            await api.fetch_book_detail(book.id)


async def test_character() -> None:
    async with ambr.AmbrAPI() as api:
        characters = await api.fetch_characters()
        for character in characters:
            await api.fetch_character_detail(character.id)


async def test_character_fetter() -> None:
    async with ambr.AmbrAPI() as api:
        characters = await api.fetch_characters()
        for character in characters:
            if "-" in character.id:
                continue
            await api.fetch_character_fetter(character.id)


async def test_food() -> None:
    async with ambr.AmbrAPI() as api:
        foods = await api.fetch_foods()
        for food in foods:
            await api.fetch_food_detail(food.id)


async def test_furniture() -> None:
    async with ambr.AmbrAPI() as api:
        furniture_sets = await api.fetch_furniture_sets()
        for furniture_set in furniture_sets:
            await api.fetch_furniture_set_detail(furniture_set.id)


async def test_material() -> None:
    async with ambr.AmbrAPI() as api:
        materials = await api.fetch_materials()
        for material in materials:
            await api.fetch_material_detail(material.id)


async def test_monster() -> None:
    async with ambr.AmbrAPI() as api:
        monsters = await api.fetch_monsters()
        for monster in monsters:
            await api.fetch_monster_detail(monster.id)


async def test_namecard() -> None:
    async with ambr.AmbrAPI() as api:
        namecards = await api.fetch_namecards()
        for namecard in namecards:
            await api.fetch_namecard_detail(namecard.id)


async def test_tcg_card() -> None:
    async with ambr.AmbrAPI() as api:
        tcg_cards = await api.fetch_tcg_cards()
        for tcg_card in tcg_cards:
            await api.fetch_tcg_card_detail(tcg_card.id)


async def test_weapon() -> None:
    async with ambr.AmbrAPI() as api:
        weapons = await api.fetch_weapons()
        for weapon in weapons:
            await api.fetch_weapon_detail(weapon.id)


async def test_artifact_sets() -> None:
    async with ambr.AmbrAPI() as api:
        artifact_sets = await api.fetch_artifact_sets()
        for artifact_set in artifact_sets:
            await api.fetch_artifact_set_detail(artifact_set.id)


async def test_furniture_sets() -> None:
    async with ambr.AmbrAPI() as api:
        furniture_sets = await api.fetch_furniture_sets()
        for furniture_set in furniture_sets:
            await api.fetch_furniture_set_detail(furniture_set.id)


async def test_achievement_categories() -> None:
    async with ambr.AmbrAPI() as api:
        await api.fetch_achievement_categories()


async def test_quests() -> None:
    async with ambr.AmbrAPI() as api:
        await api.fetch_quests()


async def test_domains() -> None:
    async with ambr.AmbrAPI() as api:
        await api.fetch_domains()


async def test_changelogs() -> None:
    async with ambr.AmbrAPI() as api:
        await api.fetch_changelogs()


async def test_upgrade_data() -> None:
    async with ambr.AmbrAPI() as api:
        await api.fetch_upgrade_data()


async def test_invalid_id() -> None:
    with pytest.raises(ambr.DataNotFoundError):
        async with ambr.AmbrAPI() as api:
            await api.fetch_character_detail("invalid")


async def test_abyss_data() -> None:
    async with ambr.AmbrAPI() as api:
        await api.fetch_abyss_data()


async def test_character_guide() -> None:
    async with ambr.AmbrAPI() as api:
        characters = await api.fetch_characters()
        for character in characters:
            await api.fetch_character_guide(character.id)
